#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  /// <summary>
  /// Linear regression data analysis algorithm.
  /// </summary>
  [Item("Radial Basis Function Regression (RBF-R)", "Radial basis function regression data analysis algorithm (uses for ALGLIB).")]
  [Creatable(CreatableAttribute.Categories.DataAnalysisRegression, Priority = 100)]
  [StorableClass]
  public sealed class RadialBasisRegression : FixedDataAnalysisAlgorithm<IRegressionProblem> {
    private const string RadialBasisRegressionModelResultName = "RBF regression solution";


    #region Parameternames
    private const string Kernelname = "Kernel";
    #endregion

    #region Paramterproperties
    public ValueParameter<IKernelFunction<double[]>> KernelParameter
    {
      get { return Parameters[Kernelname] as ValueParameter<IKernelFunction<double[]>>; }
    }
    #endregion

    #region Properties
    public IKernelFunction<double[]> Kernel
    {
      get { return KernelParameter.Value; }
    }

    #endregion

    [StorableConstructor]
    private RadialBasisRegression(bool deserializing) : base(deserializing) { }
    private RadialBasisRegression(RadialBasisRegression original, Cloner cloner)
      : base(original, cloner) {
    }
    public RadialBasisRegression() {
      Problem = new RegressionProblem();
      Parameters.Add(new ValueParameter<IKernelFunction<double[]>>(Kernelname, "The radial basis function"));
      var kernel = new PolysplineKernel<double[]>();
      KernelParameter.Value = kernel;
      kernel.BetaParameter.Value.Value = 1;
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RadialBasisRegression(this, cloner);
    }

    #region regression
    protected override void Run() {
      double loocvrmse, cvRmsError;
      var solution = CreateRadialBasisRegressionSolution(Problem.ProblemData, Kernel, out loocvrmse, out cvRmsError);
      Results.Add(new Result(RadialBasisRegressionModelResultName, "The RBF regression solution.", solution));
      Results.Add(new Result("LOOCVRMSE", "The root of the mean of squared errors of a leave-one-out-cross-validation on the trainingsset (This is not the RSME on the trainingset)", new DoubleValue(loocvrmse)));
      Results.Add(new Result("Estimated root mean square error (cross-validation)", "The estimated root of the mean of squared errors of the linear regression solution via cross validation.", new DoubleValue(cvRmsError)));
    }

    public static IConfidenceRegressionSolution CreateRadialBasisRegressionSolution(IRegressionProblemData problemData, IKernelFunction<double[]> kernel, out double loocvRmsError, out double cvRmsError) {
      var model = new RadialBasisFunctionModel(problemData.Dataset, problemData.TargetVariable, problemData.AllowedInputVariables, problemData.TrainingIndices, kernel);
      loocvRmsError = model.LeaveOneOutCrossValidationRootMeanSquaredError();
      cvRmsError = Math.Sqrt(model.GetEstimatedValues(problemData.Dataset, problemData.TestIndices)
        .Zip(problemData.TargetVariableTestValues, (a, b) => (a - b) * (a - b))
        .Sum());
      var solution = (RadialBasisFunctionRegressionSolution)model.CreateRegressionSolution((IRegressionProblemData)problemData.Clone());
      solution.Model.Name = "Radial Basis Regression Model";
      solution.Name = "Radial Basis Regression Solution";
      return solution;
    }
    #endregion

    #region helpers
    #endregion
  }
}
